/** @file Program02.cpp
 *     @brief エラーハンドリングをデモするサンプルプログラム
 *
 *     @version 1.0, 01.27.2010
 *
 *//* ***************************************************************************************** */
/** @addtogroup StatusAPI SampleProgram
 * @{
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <termios.h>
#include <unistd.h>
#include "StatusAPI.h"

#define MYASB	(ASB_NO_RESPONSE | ASB_COVER_OPEN | ASB_AUTOCUTTER_ERR | ASB_RECEIPT_NEAR_END | ASB_RECEIPT_END | ASB_UNRECOVER_ERR)

int myGetch()
{
        struct termios oldTerm, newTerm;
        int ch;

        tcgetattr( STDIN_FILENO, &oldTerm );
        newTerm = oldTerm;
        newTerm.c_lflag &= ~( ICANON | ECHO );
        tcsetattr( STDIN_FILENO, TCSANOW, &newTerm );

        do{
                ch = getchar();
        } while(ch=='\n');

        tcsetattr( STDIN_FILENO, TCSANOW, &oldTerm );

        return ch;
}

void ReadStringInput(char* strBuffer, size_t nBufferSize)
{
	/* パラメータを検証する。 */
	if((strBuffer != NULL) &&
		(nBufferSize > 0))
	{
		size_t nStrLen;
		fgets(strBuffer, nBufferSize, stdin);
		/* 文字列がNULL-終了であることを確認する。 */
		strBuffer[nBufferSize - 1] = '\0';
		/* fgetsによって自動的に付加された '\n'文字を除去する  */
		nStrLen = strlen(strBuffer);
		if(	(nStrLen > 0) &&
			('\n' == strBuffer[nStrLen - 1]))
		{
			strBuffer[nStrLen - 1] = '\0';
		}
	}
}

void PrintPrinterStatus(DWORD dwPrinterStatus)
{
	/* プリンターステータスをチェック */
	if (dwPrinterStatus & ASB_NO_RESPONSE) {
		printf("\nNo response.");
	} else if (dwPrinterStatus & ASB_COVER_OPEN) {
		printf("\nCover is open.");
	} else if (dwPrinterStatus & ASB_AUTOCUTTER_ERR) {
		printf("\nAutocutter error occured.");
	} else if (dwPrinterStatus & (ASB_RECEIPT_NEAR_END | ASB_RECEIPT_END)) {
		printf("\nRoll paper end sensor: paper not present.");
	} else if (dwPrinterStatus & ASB_UNRECOVER_ERR) {
		printf("\nAn unrecoverable error has occurred. Resetting printer");
	}
}

int main(int argc, char **argv)
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/
	CStatusAPI oStsAPI; /* ステータスAPIオブジェクト */
	int nHandle = 0; /* プリンタハンドル */
	bool bIsInitialized; /* ステータスAPIが正常に初期化されたかを示すフラッグ */
	int nType; /* プリンターオープン時のタイプ */
	char strName[128]; /* ポート名/プリンター名 */
	DWORD dwPrnStatus; /* プリンターのステータス */
	int nAPIResult; /* ステータスAPI 呼出しの結果 */
	int iKeyPressed; /* Key pressed */
	char strStringInput[64];	/* ユーザーの入力文字列 */
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/

	/* API初期化 */
	bIsInitialized = oStsAPI.Initialize();
	if (FALSE == bIsInitialized) {
		printf("Failed to open StatusAPI.\n");
		return (EXIT_FAILURE);
	}

	/* nType、strName用に、ユーザーから入力を取得する */
	do {
		printf("\nPlease enter type [TYPE_PORT(1)/TYPE_PRINTER(2)]: ");
		nType = 0;	/* タイプを初期化 */
		ReadStringInput(strStringInput, sizeof(strStringInput));	/* ユーザー入力を読み込み、メモリーに保存 */
		sscanf(strStringInput, "%d", &nType);	/* メモリーの文字列を解析し、数値を取得 */
	} while ((nType != TYPE_PORT) && (nType != TYPE_PRINTER));

	if (nType == TYPE_PORT) {
		printf("\nPlease enter port name: ");
	} else if (nType == TYPE_PRINTER) {
		printf("\nPlease enter printer name: ");
	}

	ReadStringInput(strName, sizeof(strName));

	nHandle = oStsAPI.BiOpenMonPrinter(nType, strName);

	if (nHandle <= 0) {
		printf("\nFailed to open printer status monitor.\n");
		return (EXIT_FAILURE);
	} else {
		printf("\nPrinter status monitor is started.");
	}

	printf("\nChange printer status (ex: Open Cover, Remove rolled paper....)\n");

	/* プリンター状態を確認。ユーザーが'Q'/'q'.を入力した場合ループから抜け出す */
	do {
		/* ASB_DRAWER_KICK.の間、プリンターステータスを取得 */
		do {
			nAPIResult = oStsAPI.BiGetStatus(nHandle, &dwPrnStatus);
			if (nAPIResult != SUCCESS) {
				printf("\nFailed to get status.");
				break;
			}
		} while (!(dwPrnStatus & MYASB));

		PrintPrinterStatus(dwPrnStatus);

		/* 回復不可能エラーが発生した場合プリンターをリセットする */
		if (dwPrnStatus & ASB_UNRECOVER_ERR) {
			nAPIResult = oStsAPI.BiResetPrinter(nHandle);
			if (nAPIResult != SUCCESS) {
				printf("\nFailed to reset printer.");
			}
		}

		/* ユーザーのキー入力を待つ */
		printf("\nContinue? (Q to quit/Any character to continue): ");

		iKeyPressed = toupper(myGetch());
		if(isprint(iKeyPressed))
		{
			putchar(iKeyPressed);
		}

		printf("\n");
	} while (iKeyPressed != 'Q');

	/* ステータスAPI使用後、ステータスモニターを常に閉じる */
	nAPIResult = oStsAPI.BiCloseMonPrinter(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("\nFailed to close printer status monitor.");
	}

	printf("\n");

	return (EXIT_SUCCESS);
}

/** @} **/


