/** @file Program03.cpp
 *     @brief コールバック登録をデモするサンプルプログラム
 *
 *     @version 1.0, 01.27.2010
 *
 *//* ***************************************************************************************** */
/** @addtogroup StatusAPI SampleProgram
 * @{
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "StatusAPI.h"
#include <semaphore.h>

#define MYASB	(ASB_PRINT_SUCCESS | ASB_NO_RESPONSE | ASB_AUTOCUTTER_ERR | ASB_COVER_OPEN | ASB_RECEIPT_NEAR_END | ASB_RECEIPT_END)

typedef struct _ST_USER_PARAM
{
	DWORD dwPrinterStatus;
	sem_t nMutex;
} ST_USER_PARAM;

void ReadStringInput(char* strBuffer, size_t nBufferSize)
{
	/* パラメータを検証する。 */
	if((strBuffer != NULL) &&
		(nBufferSize > 0))
	{
		size_t nStrLen;
		fgets(strBuffer, nBufferSize, stdin);
		/* 文字列がNULL-終了であることを確認する。 */
		strBuffer[nBufferSize - 1] = '\0';
		/* fgetsによって自動的に付加された '\n'文字を除去する  */
		nStrLen = strlen(strBuffer);
		if(	(nStrLen > 0) &&
			('\n' == strBuffer[nStrLen - 1]))
		{
			strBuffer[nStrLen - 1] = '\0';
		}
	}
}

void PrintPrinterStatus(DWORD dwPrinterStatus)
{
	/* 印刷完了の通知 */
	if (dwPrinterStatus & ASB_PRINT_SUCCESS) {
		printf("\nPrinting complete.");
	}
	/* エラー通知 */
	if (dwPrinterStatus & ASB_NO_RESPONSE) {
		printf("\nNo response.");
	} else if (dwPrinterStatus & ASB_COVER_OPEN) {
		printf("\nCover is open.");
	} else if (dwPrinterStatus & ASB_AUTOCUTTER_ERR) {
		printf("\nAutocutter error occurred.");
	} else if (dwPrinterStatus & (ASB_RECEIPT_NEAR_END | ASB_RECEIPT_END)) {
		printf("\nRoll paper end sensor: paper not present.");
	}
}

int CALLBACK CBGetStatus(DWORD dwStatus, LPVOID lpParam)
{
	ST_USER_PARAM* pstUserParam = (ST_USER_PARAM*)lpParam;

	/* プリンターからステータスを取得し、pstUserParaに保存する */
	pstUserParam->dwPrinterStatus = dwStatus;

	/* ステータスをチェック */
	if(dwStatus & MYASB) {
		/* セマフォにシグナルを送る */
		sem_post(&pstUserParam->nMutex);
	}

	return SUCCESS;
}

int main(int argc, char **argv)
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/
	CStatusAPI oStsAPI; /* ステータスAPIオブジェクト */
	int nHandle = 0; /* プリンタハンドル */
	bool bIsInitialized; /* ステータスAPIが正常に初期化されたかを示すフラッグ */
	int nType; /* プリンターオープン時のタイプ */
	char strName[128]; /* ポート名/プリンター名 */
	int nAPIResult; /* ステータスAPI 呼出しの結果 */
	ST_USER_PARAM stUserParam;	/* BiSetStatusBackFunctionEx2用のユーザーパラメータ */
	char strStringInput[64];	/* ユーザーの入力文字列 */
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/

	/* API初期化 */
	bIsInitialized = oStsAPI.Initialize();
	if (FALSE == bIsInitialized) {
		printf("Failed to open StatusAPI.\n");
		return (EXIT_FAILURE);
	}

	/* nType、strName用に、ユーザーから入力を取得する */
	do {
		printf("\nPlease enter type [TYPE_PORT(1)/TYPE_PRINTER(2)]: ");
		nType = 0;	/* タイプを初期化 */
		ReadStringInput(strStringInput, sizeof(strStringInput));	/* ユーザー入力を読み込み、メモリーに保存 */
		sscanf(strStringInput, "%d", &nType);	/* メモリーの文字列を解析し、数値を取得 */
	} while ((nType != TYPE_PORT) && (nType != TYPE_PRINTER));

	if (nType == TYPE_PORT) {
		printf("\nPlease enter port name: ");
	} else if (nType == TYPE_PRINTER) {
		printf("\nPlease enter printer name: ");
	}

	ReadStringInput(strName, sizeof(strName));

	nHandle = oStsAPI.BiOpenMonPrinter(nType, strName);

	if (nHandle <= 0) {
		printf("\nFailed to open printer status monitor.\n");
		return (EXIT_FAILURE);
	} else {
		printf("\nPrinter status monitor is started.");
	}

	/* stdoutに全ての文字列が表示されていることを確認 */
	fflush(stdout);

	/* セマフォの初期化 */
	if (sem_init(&stUserParam.nMutex, 0, 0) < 0) {
		printf("\nFailed to open semaphore.");
	}

	/* コールバック関数を登録 */
	nAPIResult = oStsAPI.BiSetStatusBackFunctionEx2(nHandle, CBGetStatus, &stUserParam);
	if (nAPIResult != SUCCESS) {
		printf("\nFailed to set callback function.");
	}

	/* コールバック関数からのシグナルイベントを待つ */
	sem_wait(&stUserParam.nMutex);

	PrintPrinterStatus(stUserParam.dwPrinterStatus);

	/* コールバック関数の登録を取り消す */
	nAPIResult = oStsAPI.BiCancelStatusBack(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("\nFailed to cancel callback function.");
	}

	/* ステータスAPI使用後、ステータスモニターを常に閉じる */
	nAPIResult = oStsAPI.BiCloseMonPrinter(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("\nFailed to close printer status monitor.");
	}

	/* セマフォの破棄 */
	sem_destroy(&stUserParam.nMutex);
	printf("\n");
	return (EXIT_SUCCESS);
}

/** @} **/


